// RedJey Graphics User Interface Library for J2ME Platform.
// Copyright (C) 2008 Yevgen Krapiva
//
// This library is free software; you can redistribute
// it and/or modify it under the terms of the
// GNU Lesser General Public License as published
// by the Free Software Foundation; either version 3.0
// of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

package com.incrediblesoft.redjey;

/**
 * @author Yevgen Krapiva<br>
 *         Homepage: http://code.google.com/p/redjey/<br>
 *         Feedback: ykrapiva@gmail.com<br>
 *         Copyright (C) 2008 Yevgen Krapiva<br>
 */

import java.util.Hashtable;

public class Frame extends Window
{
    protected TextComponent header;
    //protected Container contentPane;
    protected ScrollPane contentScrollPane;
    protected MenuBar menuBar;

    public Frame(String title)
    {
        this(title, null);
    }

    public Frame(String title, String styleClassName)
    {
        super(styleClassName);

        header = getDefaultHeader();
        if (header != null)
            header.parent = this;

        add(header);

        setLayout(null);
        setTitle(title);
        setState(NORMAL);
    }

    public String getName()
    {
        return "Frame";
    }

    protected Dimension calcPreferredSize(int paddingHorizontal, int paddingVertical, CustomFont font, Hashtable attributes)
    {
        return new Dimension( UIManager.getCanvas().getWidth(),
                              UIManager.getCanvas().getHeight() );
    }

    public void setTitle(String title)
    {
        if (header != null)
        {
            header.setText(title);
        }
    }

    public Container getContentPane()
    {
        if (contentScrollPane == null)
            setContentPane(new Container());
        
        return (Container) contentScrollPane.getView();
    }

    public void setContentPane(Container contentPane)
    {
        this.remove(contentScrollPane);
        contentScrollPane = new ScrollPane(contentPane);
        contentPane.setAlignment(contentPane.getAlignment() | (H_EXPAND | V_EXPAND));
        add(contentScrollPane);
        invalidate();
    }

    public void setMenuBar(MenuBar menuBar)
    {
        if (this.menuBar != null)
            this.remove(this.menuBar);

        this.menuBar = menuBar;
        insertAt(menuBar, 0);
        invalidate();
    }

    public MenuBar getMenuBar()
    {
        return menuBar;
    }

    protected TextComponent getDefaultHeader()
    {
        Label headerLabel = new Label(null);
        headerLabel.setContentAlignment(HCENTER_LAYOUT | TOP_LAYOUT);
        return headerLabel;
    }

    public Component getHeader()
    {
        return header;
    }

    public void setState(int index)
    {
        super.setState(index);
        
        if (header != null)
        {
            String headerStyle = (String) currentStyle.attributes.get("header-class");
            if (headerStyle != null) header.setStyleClass(headerStyle);
        }
        if (contentScrollPane != null)
        {
            String containerStyle = (String) currentStyle.attributes.get("content-scroll-class");
            if (containerStyle != null) contentScrollPane.getView().setStyleClass(containerStyle);
        }
        if (menuBar != null)
        {
            String softkeyBarStyle = (String) currentStyle.attributes.get("menubar-class");
            if (softkeyBarStyle != null) menuBar.setStyleClass(softkeyBarStyle);
        }

    }

    protected void validateComponent()
    {
        int totalHeight = getHeight() - currentStyle.getMinHeight();

        if (header != null)
        {
            header.setSize(new Dimension(getWidth() - currentStyle.getMinWidth(), Math.min(totalHeight, header.getPreferredSize().height)));
            header.x = 0;
            header.y = 0;
            totalHeight -= header.getHeight();
        }

        if (totalHeight > 0 && menuBar != null)
        {
            menuBar.setSize(getWidth() - currentStyle.getMinWidth(), Math.min(totalHeight, menuBar.getPreferredSize().height));
            menuBar.x = 0;
            menuBar.y = getHeight() - currentStyle.getMinHeight() - menuBar.getHeight();
            totalHeight -= menuBar.getHeight();
        }

        if (totalHeight > 0 && contentScrollPane != null)
        {
            contentScrollPane.setSize(getWidth() - currentStyle.getMinWidth(), totalHeight);
            contentScrollPane.x = 0;
            contentScrollPane.y = header != null ? header.getHeight() : 0;
            //contentPane.setFocused(true);
        }
    }

}
