// RedJey Graphics User Interface Library for J2ME Platform.
// Copyright (C) 2008 Yevgen Krapiva
//
// This library is free software; you can redistribute
// it and/or modify it under the terms of the
// GNU Lesser General Public License as published
// by the Free Software Foundation; either version 3.0
// of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

package com.incrediblesoft.redjey;

/**
 * @author Yevgen Krapiva<br>
 *         Homepage: http://code.google.com/p/redjey/<br>
 *         Feedback: ykrapiva@gmail.com<br>
 *         Copyright (C) 2008 Yevgen Krapiva<br>
 */

import com.incrediblesoft.redjey.borders.SimpleBorder;
import com.incrediblesoft.redjey.borders.SkinnedBorder;
import com.incrediblesoft.redjey.backgrounds.SimpleBackground;
import com.incrediblesoft.redjey.event.PaintEvent;
import com.incrediblesoft.redjey.event.MouseEvent;
import com.incrediblesoft.redjey.event.FocusEvent;
import com.incrediblesoft.redjey.event.AbstractEvent;

import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.Sprite;
import javax.microedition.midlet.MIDlet;
import java.util.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;


public class UIManager
{
    /*
     * Key code for left soft key
     */
    public static int KEY_SOFT_LEFT = -6;
    /*
     * Key code for right soft key
     */
    public static int KEY_SOFT_RIGHT = -7;
    /**
     * Key code for selection key
     */
    public static int KEY_SELECT = -5;
    /**
     * Key code for the LEFT game action.
     */
    public static int KEY_LEFT = -3;
    /**
     * Key code for the RIGHT game action.
     */
    public static int KEY_RIGHT = -4;
    /**
     * Key code for the UP game action.
     */
    public static int KEY_UP = -1;
    /**
     * Key code for the DOWN game action.
     */
    public static int KEY_DOWN = -2;
    /**
     * Key code for CLEAR key
     */
    public static int KEY_CLEAR = -8;

    /**
     * Key pressed flags
     */
    public static boolean leftKeyPressed;
    public static boolean rightKeyPressed;
    public static boolean upKeyPressed;
    public static boolean downKeyPressed;
    public static boolean selectKeyPressed;

    //LOCALIZATION STRINGS
    public static String APPLY_CMD = "Apply";
    public static String MENU_CMD = "Menu";
    public static String CANCEL_CMD = "Cancel";

    //Internal stuff
    private static Vector windows = new Vector();
    private static Display display;
    private static Canvas canvas;
    private static MIDlet midlet;
    private static Hashtable resHash = new Hashtable();
    protected static TextBox box;
    protected static Command applyCmd, cancelCmd;
    private static Vector paintEventQueue = new Vector();
    private static Vector eventQueue = new Vector();
    private static Window prevFocusedWindow, prevEnteredWindow;
    private static Component prevTargetComponent, prevEnteredComponent;

    //Cursor stuff
    private static CustomImage cursorImage;
    private static boolean cursorVisible = true;
    private static int curX, curY, prevCurX, prevCurY;
    private static int cursorSpeed, maxCursorSpeed, cursorAcc;
    private static boolean mouseScrolling;
    //private static int mouseScrollType;
    private static int dxScroll, dyScroll;
    private static long leftKeyReleasedTime, rightKeyReleasedTime, upKeyReleasedTime, downKeyReleasedTime;
    private static int mouseScrollSensivity = 200;
    private static int backgroundColor;
    private static boolean isBackgroundColorSet;
    

    //CACHED STYLES
    private static Hashtable styles_hash = new Hashtable();
    //Predefined style sets
    public static final String DEFAULT_SET = "default";
    //String representation of component state
    private static final String NORMAL_STR = "normal";
    private static final String HIGHLIGHTED_STR = "highlighted";
    private static final String PRESSED_STR = "pressed";
    private static final String DISABLED_STR = "disabled";

    //Repeat stuff
    private static final Hashtable repeatTaskTable = new Hashtable();

    private UIManager()
    {
    }

    /**
     * Assignes MIDlet class. Neccessary for operating.
     * @param midlet_
     */
    public static void setMIDlet(MIDlet midlet_)
    {
        midlet = midlet_;
        display = Display.getDisplay(midlet);
        box = new TextBox(null, null, 255, javax.microedition.lcdui.TextField.ANY);
        applyCmd = new Command(APPLY_CMD, Command.OK, 1);
        cancelCmd = new Command(CANCEL_CMD, Command.CANCEL, 1);
        box.addCommand(applyCmd);
        box.addCommand(cancelCmd);
    }

    public static MIDlet getMIDlet()
    {
        return midlet;
    }

    public static Display getDisplay()
    {
        return display;
    }

    public static Canvas getCanvas()
    {
        return canvas;
    }

    public static void setCanvas(Canvas canvas_)
    {
        canvas = canvas_;
        //cursorAcc = 15;
        //maxCursorSpeed = 10;
        cursorAcc = canvas.getWidth() * canvas.getHeight() / 4960;
        maxCursorSpeed = canvas.getWidth() * canvas.getHeight() / 7000;
    }

    public static void setCursorImage(CustomImage cursorImage_)
    {
        cursorImage = cursorImage_;
    }

    public static CustomImage getCursorImage()
    {
        return cursorImage;
    }

    public static void setCursorVisible(boolean b)
    {
        cursorVisible = b;
    }

    public static void setCursorPos(int x, int y)
    {
        setCursorPosX(x);
        setCursorPosY(y);
    }

    public static void setCursorPosX(int x)
    {
        prevCurX = curX;
        curX = Math.max(0, Math.min(canvas.getWidth()-1, x));
    }

    public static void setCursorPosY(int y)
    {
        prevCurY = curY;
        curY = Math.max(0, Math.min(canvas.getHeight()-1, y));
    }

    public static int getCursorPosX()
    {
        return curX;
    }

    public static int getCursorPosY()
    {
        return curY;
    }

    public static void postEvent(AbstractEvent event)
    {
        eventQueue.addElement(event);
    }

    public static void addWindow(Window window)
    {
        insertWindowAt(window, windows.size());
    }

    public static void insertWindowAt(Window window, int index)
    {
        windows.removeElement(window);

        //Insert new window
        windows.insertElementAt(window, index);

        if (index == 0)
        {
            if (prevFocusedWindow != window)
            {
                window.dispatchEvent(new FocusEvent(window, FocusEvent.FOCUS_GAINED));
                if (prevFocusedWindow != null)
                {
                    prevFocusedWindow.dispatchEvent(new FocusEvent(prevFocusedWindow, FocusEvent.FOCUS_LOST));
                }
                prevFocusedWindow = window;
            }
        }
        window.repaint();
    }

    public static int indexOf(Window window)
    {
        return windows.indexOf(window);
    }

    public static void removeWindow(Window window)
    {
        int index = windows.indexOf(window);
        if (index == -1) return;

        windows.removeElementAt(index);
        if (windows.isEmpty()) prevFocusedWindow = null;

        repaint();
    }

    /*public static void setZOrder(Window window, int newIndex)
    {
        int index = indexOf(window);
        if (index != -1)
        {
            windows.removeElementAt(index);
            insertWindowAt(window, newIndex);
        }
    }*/

    public static int getNumWindows()
    {
        return windows.size();
    }
    
    /**
     * Resets screen stack
     */
    public static void removeAllWindows()
    {
        windows.removeAllElements();
    }

    public static void postPaintEvent(PaintEvent event)
    {
        //System.out.println("postPaintEvent source "+event.getSource()+"  rect: "+event.getUpdateRect());

        /*int index = 0;
        while ((index = queueContainsSource(index, event.getSource())) != -1)
        {
            Rectangle newRect = mergePaintUpdateEvents(event, (PaintEvent) paintEventQueue.elementAt(index));
            if (newRect != null)
            {
                event.setUpdateRect(newRect);
                paintEventQueue.removeElementAt(index);
            }
            else
                index++;
        }*/
       // int index = windows.indexOf(event.getSource());
       // if (index != -1)
        //{
            paintEventQueue.addElement(event);
        /*    for ( --index; index >= 0; index--)
            {
                Window windowOnTop = (Window) windows.elementAt(index);
                Rectangle intersectedRect = event.getUpdateRect().intersection(windowOnTop.getBounds());
                if (intersectedRect )
            }
        } */
    }

    private static Rectangle mergePaintUpdateEvents(PaintEvent a, PaintEvent b)
    {
        Rectangle aRect = a.getUpdateRect();
        Rectangle bRect = b.getUpdateRect();
        if (bRect.contains(aRect))
        {
            return bRect;
        }
        if (aRect.contains(bRect))
        {
            return aRect;
        }
        return null;
    }

    private static int queueContainsSource(int fromIndex, Component source)
    {
        for (int i=fromIndex; i< paintEventQueue.size(); i++)
            if (((PaintEvent) paintEventQueue.elementAt(i)).getSource() == source)
                return i;
        return -1;
    }

    /**
     * Called every frame.
     * @param dt milliseconds passed from previous call
     */
    public static void tick(int dt)
    {
        //if direction keys pressed then change cursor speed
        if (leftKeyPressed | rightKeyPressed | upKeyPressed | downKeyPressed)
        {
            cursorSpeed += (cursorAcc * dt << 8) / 1000;
            cursorSpeed = Math.min(cursorSpeed, maxCursorSpeed << 8);
        }
        //else stop cursor
        else
        {
            //cursorSpeed -= (4 * dt << 8) / 1000;
            //cursorSpeed = Math.max(cursorSpeed, 0);
            cursorSpeed = 0;
            mouseScrolling = false;
        }

        if (!mouseScrolling)
        {
            //Set new cursor position
            if (leftKeyPressed)
                setCursorPosX(curX - (cursorSpeed >> 8));
            else if (rightKeyPressed)
                setCursorPosX(curX + (cursorSpeed >> 8));
            if (upKeyPressed)
                setCursorPosY(curY - (cursorSpeed >> 8));
            else if (downKeyPressed)
                setCursorPosY(curY + (cursorSpeed >> 8));
        }
        //if scrolling feature enabled, then send events about it
        if (mouseScrolling)
        {
            if (dxScroll != 0)
                postEvent(new MouseEvent(null, MouseEvent.MOUSE_SCROLL_HORIZONTAL, curX, curY, dxScroll));
            if (dyScroll != 0)
                postEvent(new MouseEvent(null, MouseEvent.MOUSE_SCROLL_VERTICAL, curX, curY, dyScroll));
            postEvent(new MouseEvent(null, MouseEvent.MOUSE_MOVED, curX , curY, 0));
        }
        //otherwise send other events
        else
        {
            if (curX != prevCurX | curY != prevCurY)
            {
                postEvent(new MouseEvent(null, MouseEvent.MOUSE_MOVED, curX , curY, 0));
                if (cursorVisible)
                {
                    postPaintEvent(new PaintEvent(null, new Rectangle(prevCurX, prevCurY, cursorImage.getWidth(), cursorImage.getHeight())));
                }
            }
           // else if (selectKeyPressed)
           //     postEvent(new MouseEvent(null, MouseEvent.MOUSE_PRESSED, curX, curY, 0));
        }

        processEventQueue();

        for (int i=0; i<windows.size(); i++)
        {
            ((Window) windows.elementAt(i)).tick(dt);
        }

        prevCurX = curX;
        prevCurY = curY;
    }

    private static void processEventQueue()
    {
        //Dispatch all not consumed events to destintations
        for (int i=0; i< eventQueue.size(); i++)
        {
            AbstractEvent event = (AbstractEvent) eventQueue.elementAt(i);
            if (event.isConsumed()) continue;

            //Get source
            Component source = event.getSource();
            //If source object is defined
            if (source != null)
            {
                //Dispatch event directly to source
                source.dispatchEvent(event);
            }
            //Else if source not defined and this is a MousEvent
            else if (event.getEventType() == AbstractEvent.MOUSE_EVENT)
            {
                MouseEvent mouseEvent = (MouseEvent) event;

                //Look through each window
                for (int n=0; n<windows.size(); n++)
                {
                    Window window = (Window) windows.elementAt(n);

                    //Find window under the mouse cursor position
                    if (window.getBounds().inside(mouseEvent.getX(), mouseEvent.getY()))
                    if (!(prevFocusedWindow != window && prevFocusedWindow.isModal()))
                    {
                        //Translate mouse coords into window coordinate system
                        mouseEvent = retargetMouseEvent(mouseEvent, window);

                        //Check different event types
                        switch(event.getID())
                        {
                            case MouseEvent.MOUSE_PRESSED:
                                //Remember the object over which mouse was pressed, to generate later MOUSE_CLICKED event
                                prevTargetComponent = window.getComponentAt(mouseEvent.getX(), mouseEvent.getY(), true);
                                //System.out.println("prevTargetComponent is "+prevTargetComponent+"  with mouse coords ("+mouseEvent.getX()+","+mouseEvent.getY()+")");

                                //insertWindowAt() replaces prevFocusedWindow
                                Window backupPrevWindow = prevFocusedWindow;

                                //Bring the new window to top
                                insertWindowAt(window, 0);

                                //From now the current window is the new window.
                                //If the previuos window was a PopupMenu and the new window doesn't belong
                                //to menues in any way, we need to close opened menues.
                                if (backupPrevWindow instanceof PopupMenu && !(window instanceof PopupMenu))
                                {
                                    //Component targetComponent = window.getComponentAt(mouseEvent.getX(), mouseEvent.getY(), true);
                                    //System.out.println("targetComponent = "+targetComponent);
                                    PopupMenu popup = (PopupMenu) backupPrevWindow;
                                    Component root = popup.getRoot();
                                    if (root != prevTargetComponent)
                                    {
                                        if (root instanceof PopupMenu)
                                            ((PopupMenu) root).setVisible(false);
                                        else if (root instanceof Menu)
                                            ((Menu) root).close();
                                    }

                                }

                                window.dispatchEvent(mouseEvent);

                                break;

                            case MouseEvent.MOUSE_MOVED:
                                //Remember the object over which mouse was pressed, to generate later MOUSE_CLICKED event
                                Component currentEnteredComponent = window.getComponentAt(mouseEvent.getX(), mouseEvent.getY(), true);
                                //System.out.println("prevEnteredComponent is "+prevEnteredComponent+" and current is "+currentEnteredComponent);
                                if (prevEnteredComponent != currentEnteredComponent)
                                {
                                    if (prevEnteredComponent != null)
                                    {
                                        prevEnteredComponent.dispatchEvent(new MouseEvent(prevEnteredComponent, MouseEvent.MOUSE_EXITED));
                                    }
                                    if (currentEnteredComponent == null)
                                    currentEnteredComponent = window;
                                    
                                    currentEnteredComponent.dispatchEvent(new MouseEvent(currentEnteredComponent, MouseEvent.MOUSE_ENTERED));

                                    prevEnteredComponent = currentEnteredComponent;
                                }

                                //If mouse moved to another window
                                /*if (prevEnteredWindow != window)
                                {
                                    //Send MOUSE_EXITED event to previous window
                                    if (prevEnteredWindow != null)
                                        prevEnteredWindow.dispatchEvent(new MouseEvent(prevEnteredWindow, MouseEvent.MOUSE_EXITED));
                                    //Send MOUSE_ENTERED event to the current one
                                    window.dispatchEvent(new MouseEvent(window, MouseEvent.MOUSE_ENTERED));
                                    prevEnteredWindow = window;
                                }                                */
                                //At last send the original event
                                window.dispatchEvent(mouseEvent);
                                break;

                            case MouseEvent.MOUSE_RELEASED:
                                //First, send the original event
                                //System.out.println("AAAAA ("+mouseEvent.getX()+","+mouseEvent.getY()+")");
                                //At last send the original event
                                window.dispatchEvent(mouseEvent);

                                //Find targeted component looking to the deep of tree
                                Component targetComponent = window.getComponentAt(mouseEvent.getX(), mouseEvent.getY(), true);
                                //System.out.println("targetComponent is "+targetComponent+"  with mouse coords ("+mouseEvent.getX()+","+mouseEvent.getY()+")");
                                //if targeted component is the same it was in MOUSE_PRESSED event, send MOUSE_CLICKED event to it.
                                if (targetComponent == prevTargetComponent)
                                    window.dispatchEvent(new MouseEvent(window, MouseEvent.MOUSE_CLICKED, mouseEvent.getX(), mouseEvent.getY(), 0));

                                prevTargetComponent = null;
                                break;

                            default:
                                //At last send the original event
                                window.dispatchEvent(mouseEvent);
                        }
                        break;
                    }
                }
            }
        }

        //clear event queue
        eventQueue.removeAllElements();
    }

    public static MouseEvent retargetMouseEvent(MouseEvent origEvent, Component newSource)
    {
        int new_x = origEvent.getX() - newSource.getX() - (newSource.parent != null ? newSource.parent.getCurrentStyle().getClientOffsetX() : 0);
        int new_y = origEvent.getY() - newSource.getY() - (newSource.parent != null ? newSource.parent.getCurrentStyle().getClientOffsetY() : 0);

        MouseEvent newEvent = new MouseEvent(newSource, origEvent.getID(), new_x, new_y, origEvent.getValue());
        return newEvent;
    }

    public synchronized static void keyPressed(int keycode)
    {
        boolean keyConsumed = false;

        if (keycode == KEY_LEFT)
        {
            leftKeyPressed = true;
            if (System.currentTimeMillis() - leftKeyReleasedTime < mouseScrollSensivity)
            {
                mouseScrolling = true;
                //mouseScrollType = MouseEvent.MOUSE_SCROLL_LEFT;
                dxScroll = -5;
                dyScroll = 0;
            }
        }
        else if (keycode == KEY_RIGHT)
        {
            rightKeyPressed = true;
            if (System.currentTimeMillis() - rightKeyReleasedTime < mouseScrollSensivity)
            {
                mouseScrolling = true;
                //mouseScrollType = MouseEvent.MOUSE_SCROLL_RIGHT;
                dxScroll = 5;
                dyScroll = 0;
            }
        }
        if (keycode == KEY_UP)
        {
            upKeyPressed = true;
            if (System.currentTimeMillis() - upKeyReleasedTime < mouseScrollSensivity)
            {
                mouseScrolling = true;
                //mouseScrollType = MouseEvent.MOUSE_SCROLL_UP;
                dyScroll = -5;
                dxScroll = 0;
            }
        }
        else if (keycode == KEY_DOWN)
        {
            downKeyPressed = true;
            if (System.currentTimeMillis() - downKeyReleasedTime < mouseScrollSensivity)
            {
                mouseScrolling = true;
                //mouseScrollType = MouseEvent.MOUSE_SCROLL_DOWN;
                dyScroll = 5;
                dxScroll = 0;
            }
        }   
        if (keycode == KEY_SELECT)
        {
            postEvent(new MouseEvent(null, MouseEvent.MOUSE_PRESSED, curX, curY, 0));
            selectKeyPressed  = true;
        }
    }

    
    public synchronized static void keyRepeated(int keycode)
    {
        if (keycode == KEY_SELECT)
        {
            postEvent(new MouseEvent(null, MouseEvent.MOUSE_PRESSED, curX, curY, 0));
            selectKeyPressed  = true;
        }
    }

    public synchronized static void keyReleased(int keycode)
    {
        if (keycode == KEY_LEFT)
        {
            leftKeyPressed = false;//System.out.println("left");
            leftKeyReleasedTime = System.currentTimeMillis();
        }
        else if (keycode == KEY_RIGHT)
        {
            rightKeyPressed = false;//System.out.println("right");
            rightKeyReleasedTime = System.currentTimeMillis();
        }
        if (keycode == KEY_UP)
        {
            upKeyPressed = false;//System.out.println("up");
            upKeyReleasedTime = System.currentTimeMillis();
        }
        else if (keycode == KEY_DOWN)
        {
            downKeyPressed = false;//System.out.println("down");
            downKeyReleasedTime = System.currentTimeMillis();
        }
        if (keycode == KEY_SELECT)
        {
            if (selectKeyPressed == true)
            {
                postEvent(new MouseEvent(null, MouseEvent.MOUSE_RELEASED, curX, curY, 0));
                selectKeyPressed  = false;
            }
        }

        //if (currentScreen != null)
        //    currentScreen.keyReleased(keycode);  */
    }

    public static void pointerPressed(int x, int y)
    {
        //System.out.println("pointerPressed("+x+","+y+")");
        setCursorPos(x, y);
        postEvent(new MouseEvent(null, MouseEvent.MOUSE_PRESSED, x, y, 0));
        postEvent(new MouseEvent(null, MouseEvent.MOUSE_MOVED, curX , curY, 0));
    }

    public static void pointerReleased(int x, int y)
    {
        //System.out.println("pointerReleased("+x+","+y+")");
        setCursorPos(x, y);
        postEvent(new MouseEvent(null, MouseEvent.MOUSE_RELEASED, x, y, 0));
        postEvent(new MouseEvent(null, MouseEvent.MOUSE_MOVED, curX , curY, 0));
    }

    public static void pointerDragged(int x, int y)
    {
        int dx = curX - x;
        int dy = curY - y;
        setCursorPos(x, y);
        //System.out.println("dx="+dx+"  dy="+dy);
        if (dx != 0)
            postEvent(new MouseEvent(null, MouseEvent.MOUSE_SCROLL_HORIZONTAL, curX, curY, dx));
        if (dy != 0)
            postEvent(new MouseEvent(null, MouseEvent.MOUSE_SCROLL_VERTICAL, curX, curY, dy));
        postEvent(new MouseEvent(null, MouseEvent.MOUSE_MOVED, curX , curY, 0));
    }

    public static void repaint()
    {
        postPaintEvent(new PaintEvent(null, new Rectangle(0, 0, canvas.getWidth(), canvas.getHeight())));
    }

    public static boolean isCursorMoving()
    {
        return curX != prevCurX | curY != prevCurY;
    }

    public static void showNotify()
    {
        leftKeyPressed = rightKeyPressed = upKeyPressed = downKeyPressed = selectKeyPressed = false;
    }

    public static void hideNotify()
    {
        leftKeyPressed = rightKeyPressed = upKeyPressed = downKeyPressed = selectKeyPressed = false;
    }

    private synchronized static void processPaintEventQueue(Graphics g)
    {
        int oldTransX;
        int oldTransY;
        int oldClipX;
        int oldClipY;
        int oldClipWidth;
        int oldClipHeight;

        for (int i=0; i< paintEventQueue.size(); i++)
        {
            PaintEvent event = (PaintEvent) paintEventQueue.elementAt(i);

            Window source = (Window) event.getSource();

            int index = windows.indexOf(source);
            if (index == -1) index = windows.size()-1;

            Rectangle absoluteUpdateRect = event.getUpdateRect();
            if (source != null)
                absoluteUpdateRect.setLocation(source.x + absoluteUpdateRect.x, source.y + absoluteUpdateRect.y);

            for ( ; index >= 0; index--)
            {
                Window windowOnTop = (Window) windows.elementAt(index);
                Rectangle rect = windowOnTop.getBounds().intersection(absoluteUpdateRect);

                if (!rect.isEmpty())
                {
                    //System.out.println("Painting "+windowOnTop.toString()+"  at "+rect+" -> "+System.currentTimeMillis());

                    oldTransX = g.getTranslateX();
                    oldTransY = g.getTranslateY();
                    oldClipX = g.getClipX();
                    oldClipY = g.getClipY();
                    oldClipWidth = g.getClipWidth();
                    oldClipHeight = g.getClipHeight();

                    rect.x -= windowOnTop.x;
                    rect.y -= windowOnTop.y;

                    g.translate(windowOnTop.getX(), windowOnTop.getY());
                    g.setClip(rect.getX(), rect.getY(), rect.getWidth(), rect.getHeight());
                    //dbg.setClip(0, 0, rect.getWidth(), rect.getHeight());

                    //System.out.println("Painting "+source+"  at "+new Date());
                    windowOnTop.paint(g);

                    g.translate(oldTransX - g.getTranslateX(), oldTransY - g.getTranslateY());
                    g.setClip(oldClipX, oldClipY, oldClipWidth, oldClipHeight);
                }
            }
        }

        paintEventQueue.removeAllElements();
    }

    public static void setBackgroundColor(int color)
    {
        backgroundColor = color;
        isBackgroundColorSet = true;
    }

    public static void switchBackgroundColor(boolean enable)
    {
        isBackgroundColorSet = enable;
    }

    public synchronized static void paint(Graphics g)
    {
        if (paintEventQueue.size() > 0)
        {
            if (isBackgroundColorSet)
            {
                g.setColor(backgroundColor);
                g.fillRect(0, 0, canvas.getWidth(), canvas.getHeight());
            }
            
            processPaintEventQueue(g);

            if (cursorVisible && cursorImage != null)
                cursorImage.paint(g, curX, curY, Graphics.LEFT | Graphics.TOP);
        }
    }

     /*
     * ***********************************************************************************
     * ***********************************************************************************
     *                                 U   T   I   L   S
     * ***********************************************************************************
     * ***********************************************************************************
     */

    public static void cacheImage(String imagePath) throws IOException
    {
        if (!resHash.containsKey(imagePath))
        {
            resHash.put(imagePath, Image.createImage(imagePath));
        }
    }

    public static Image getImage(String imagePath)
    {
        return (Image) resHash.get(imagePath);
    }

    public static CustomImage getCustomImage(String imagePath)
    {
        return new CustomImage((Image) resHash.get(imagePath));
    }

    public static void removeAllRegisteredStyles()
    {
        styles_hash.clear();
    }

    public static Hashtable getStylesScheme()
    {
        return styles_hash;
    }

    public static void setStylesScheme(Hashtable scheme)
    {
        styles_hash = scheme;
    }

    /**
     * Registers style class
     * @param componentName component name. See Component.getName()
     * @param className class name. The default is UIManager.DEFAULT_SET
     * @param styles Styles array. Each element in array corresponds to the specific state of component. See Component.getState()
     */
    public static void registerStyleClass(String componentName, String className, Style[] styles)
    {
        if (!styles_hash.containsKey(componentName))
        {
            styles_hash.put(componentName, new Hashtable());
        }

        Hashtable styleClassHash = (Hashtable) styles_hash.get(componentName);
        if (styleClassHash != null)
        {
            styleClassHash.put(className, styles);
        }
    }

    /**
     * Registers single style
     * @param componentName Component name. see Component.getName()
     * @param className Class name to register the style in.
     * @param componentState Component state.
     * @param style Style itself.
     */
    public static void registerStyle(String componentName, String className, int componentState, Style style)
    {
        boolean reput = false;

        if (!styles_hash.containsKey(componentName))
        {
            styles_hash.put(componentName, new Hashtable());
        }

        Hashtable styleSetHash = (Hashtable) styles_hash.get(componentName);
        if (!styleSetHash.containsKey(className))
        {
            styleSetHash.put(className, new Style[1]);
        }

        Style[] styles = (Style[]) styleSetHash.get(className);
        if (styles.length <= componentState)
        {
            Style[] oldStyles = styles;
            styles = new Style[componentState+1];
            for (int i=0; i<oldStyles.length; i++)
                styles[i] = oldStyles[i];
            reput = true;
        }
        styles[componentState] = style;
        if (reput)
            styleSetHash.put(className, styles);
    }

    public static Style[] getStyleClass(String componentName)
    {
        return getStyleClass(componentName, DEFAULT_SET);
    }

    public static Style[] getStyleClass(String componentName, String className)
    {
        Hashtable styleClassHash = (Hashtable) styles_hash.get(componentName);
        Style[] styleSet = null;

        if (styleClassHash != null)
        {
            styleSet = (Style[]) styleClassHash.get(className);
        }

        if (styleSet == null && !componentName.equals("Component"))
            styleSet = getStyleClass("Component", className);

        if (styleSet != null)
        {
            Style[] clonedSet = new Style[styleSet.length];
            for (int i=0; i<styleSet.length; i++)
                if (styleSet[i] != null) clonedSet[i] = styleSet[i].clone();
            styleSet = clonedSet;
        }
        else
            System.out.println("Cannot find style class "+componentName+"."+className);


        return styleSet;
    }

    public static Style getStyle(String componentName, String className, int componentState)
    {
        Style[] styleSet = getStyleClass(componentName, className);
        if (styleSet != null && styleSet.length > componentState)
            return styleSet[componentState].clone();
        else
            return null;
    }

    /**
     * Parses styles description file and registers all styles listed in that
     * @param path File path
     * @param printDebugInfo  Prints debug info to System.out
     * @throws IOException As usual
     */
    public static void loadStylesFromResource(String path, boolean printDebugInfo) throws IOException
    {
        InputStream is = null;
        ByteArrayOutputStream baos = null;

        try
        {
            is = UIManager.getMIDlet().getClass().getResourceAsStream(path);
            baos = new ByteArrayOutputStream();

            int n;
            byte[] buf = new byte[1024];
            while ((n = is.read(buf)) != -1)
                baos.write(buf, 0, n);

            baos.close();
            is.close();
            buf = null;
            
            String text = new String(baos.toByteArray());
            loadStyles(text, printDebugInfo);
        }
        finally
        {
            if (is != null) try {is.close();} catch (IOException ex) { /* ignore */ }
            if (baos != null) try {baos.close();} catch (IOException ex) { /* ignore */ }
        }

        baos = null;
        is = null;
    }

    public static void loadStyles(String temptext, boolean printDebugInfo)
    {
        StringBuffer buf = new StringBuffer();
        String line;
        int index, fromIndex = 0;

        while ((index = temptext.indexOf('\n', fromIndex)) != -1)
        {
            line = temptext.substring(fromIndex, index).trim();
            if (!line.startsWith("#")) buf.append(line).append("\n");
            fromIndex = index+1;
        }

        line = temptext.substring(fromIndex).trim();
        if (!line.startsWith("#")) buf.append(line);

        String text = buf.toString();
        buf = null;
        line = null;

        fromIndex = 0;

        //System.out.println(text);

        while ((index = text.indexOf('{', fromIndex)) != -1)
        {
            String styleName;
            String className;
            int styleState;

            String definition = text.substring(fromIndex, index).trim();
            Object[] extendedClassDesc = null;

            int extendsIndex = definition.indexOf("extends");
            if (extendsIndex != -1)
            {
                extendedClassDesc = parseStyleDescription(definition.substring(extendsIndex+7).trim());
                definition = definition.substring(0, extendsIndex).trim();
            }

            Object[] thisClassDesc = parseStyleDescription(definition);
            styleName = (String) thisClassDesc[0];
            className = (String) thisClassDesc[1];
            styleState = ((Integer) thisClassDesc[2]).intValue();

            Style style = null;

            if (extendedClassDesc != null)
            {
                Style extendedStyle = getStyle((String) extendedClassDesc[0], (String) extendedClassDesc[1], ((Integer) extendedClassDesc[2]).intValue());
                if (extendedStyle != null)
                {
                    style = extendedStyle.clone();
                }
            }

            if (style == null)
                style = new Style(null, null, null, null);

            if (printDebugInfo)
                System.out.println(styleName+"."+className+":"+styleState);

            int index2 = indexOfIgnoringBrackets(text, '}', index+1);

            if (!styleName.startsWith("#"))
            {
                Hashtable desc = (Hashtable) parseValue(text, index, index2);
                Enumeration en = desc.keys();

                while (en.hasMoreElements())
                {
                    String name = (String) en.nextElement();
                    Object value = desc.get(name);

                    if (printDebugInfo)
                        System.out.println("   "+name+":"+value);

                    if (name.equals("padding-left"))
                        style.paddingLeft = Integer.parseInt((String) value);
                    else if (name.equals("padding-right"))
                        style.paddingRight = Integer.parseInt((String) value);
                    else if (name.equals("padding-top"))
                        style.paddingTop = Integer.parseInt((String) value);
                    else if (name.equals("padding-bottom"))
                        style.paddingBottom = Integer.parseInt((String) value);
                    else if (name.equals("padding-vertical"))
                        style.paddingVertical = Integer.parseInt((String) value);
                    else if (name.equals("padding-horizontal"))
                        style.paddingHorizontal = Integer.parseInt((String) value);
                    else if (name.equals("margin-left"))
                        style.marginLeft = Integer.parseInt((String) value);
                    else if (name.equals("margin-right"))
                        style.marginRight = Integer.parseInt((String) value);
                    else if (name.equals("margin-top"))
                        style.marginTop = Integer.parseInt((String) value);
                    else if (name.equals("margin-bottom"))
                        style.marginBottom = Integer.parseInt((String) value);
                    else
                    {
                        if (name.equals("background-simple"))
                        {
                            style.background = new SimpleBackground((Hashtable)value);
                        }
                        else if (name.equals("border-simple"))
                        {
                            style.border = new SimpleBorder((Hashtable)value);
                        }
                        else if (name.equals("border-skin"))
                        {
                            style.border = new SkinnedBorder((Hashtable) value);
                        }
                        else if (name.equals("font"))
                        {
                            style.font = CustomFont.getFont(value);
                        }
                        else
                        {
                            style.attributes.put(name, value);
                        }
                    }
                }

                /*Style style = new Style(margin_left, margin_right, margin_top, margin_bottom,
                                        padding_left, padding_right, padding_top, padding_bottom,
                                        padding_horizontal, padding_vertical,
                                        font,
                                        background,
                                        border,
                                        attributes);
                */
                registerStyle(styleName, className, styleState, style);
            }
            fromIndex = index2+1;
        }
    }

    private static Object[] parseStyleDescription(String s)
    {
        String componentName;
        String className = DEFAULT_SET;
        String componentStateStr = NORMAL_STR;
        String definition = s;

        int stateIndex = definition.indexOf(':');
        if (stateIndex != -1)
        {
            componentStateStr = definition.substring(stateIndex+1).trim();
            definition = definition.substring(0, stateIndex).trim();
        }

        int classIndex = definition.indexOf('.');
        if (classIndex != -1)
        {
            className = definition.substring(classIndex+1).trim();
            definition = definition.substring(0, classIndex).trim();
        }

        componentName = definition;


        int styleState;

        if (componentStateStr.equals(NORMAL_STR))
            styleState = Component.NORMAL;
        else if (componentStateStr.equals(HIGHLIGHTED_STR))
            styleState = Component.HIGHLIGHTED;
        else if (componentStateStr.equals(PRESSED_STR))
            styleState = Component.PRESSED;
        else if (componentStateStr.equals(DISABLED_STR))
            styleState = Component.DISABLED;
        else
            styleState = Integer.parseInt(componentStateStr);

        return new Object[] {componentName, className, new Integer(styleState)};

    }

    public static Object parseValue(String s)
    {
        return parseValue(s, 0, s.length());
    }

    /**
     * Parses some value
     * @param s Text that represents value
     * @param fromIndex Index to start with
     * @param toIndex Index to end with
     * @return Hashtable if value has nested attributes, else returns the same 's' variable that was mentioned above
     */
    public static Object parseValue(String s, int fromIndex, int toIndex)
    {
        Hashtable table;

        //if value starts with bracket
        if (s.charAt(fromIndex) == '{')
        {
            table = new Hashtable();
            fromIndex++;

            int indexAttr = 0;
            //Move along all attributes
            while ((indexAttr = indexOfIgnoringBrackets(s, ';', fromIndex)) != -1 && indexAttr < toIndex)
            {
                String attribute = s.substring(fromIndex, indexAttr).trim();
                int indexDelimeter = attribute.indexOf(':');

                //Getting name and value
                String name = attribute.substring(0, indexDelimeter).trim();
                String value = attribute.substring(indexDelimeter+1).trim();
                if (!name.startsWith("#"))
                {
                    //Recursively parse value and put this attribute to hashtable
                    Object parsedValue = parseValue(value);

                    //Recognize common objects
                    if (name.equals("image"))
                    {
                        parsedValue = new CustomImage((Hashtable) parsedValue);
                    }

                    table.put(name, parsedValue);
                }
                //else
                //System.out.println("skipped "+name+" : "+value);                
                
                fromIndex = indexAttr+1;
            }

            return table;
        }
        else
        {
            String attribute = s.substring(fromIndex);
            int indexDelimeter = attribute.indexOf(':');
            Object parsedValue = null;

            if (indexDelimeter != -1)
            {
                String name = attribute.substring(0, indexDelimeter).trim();
                String value = attribute.substring(indexDelimeter+1).trim();
                if (name.equals("image")) parsedValue = new CustomImage((Hashtable) parseValue(value));
            }
            else parsedValue = s;

            return parsedValue;
        }
    }

    /**
     * Searches for character in the { .. } scope
     * @param s Text to search in
     * @param ch Character to find
     * @param fromIndex Index to begin with
     * @return Index of character, or -1 if it was not found in the current scope.
     */
    public static int indexOfIgnoringBrackets(String s, char ch, int fromIndex)
    {
        int bracketsCount = 0;
        char c;

        for (int i=fromIndex; i<s.length(); i++)
        {
            c = s.charAt(i);
            if (c == ch && bracketsCount == 0)
                return i;
            else if (c == '{')
                bracketsCount++;
            else if (c == '}')
                bracketsCount--;
        }

        return -1;
    }

    public static int floor(int a, int b)
    {
        int c = a / b;
        if (a % b > 0) c++;
        return c;
    }


    private class RepeatTask extends TimerTask
    {
        private int keycode;

        public RepeatTask(int keycode)
        {
            this.keycode = keycode;
        }

        public void run()
        {
            keyRepeated(keycode);
        }
    }

}
