// RedJey Graphics User Interface Library for J2ME Platform.
// Copyright (C) 2008 Yevgen Krapiva
//
// This library is free software; you can redistribute
// it and/or modify it under the terms of the
// GNU Lesser General Public License as published
// by the Free Software Foundation; either version 3.0
// of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

package com.incrediblesoft.redjey;

/**
 * @author Yevgen Krapiva<br>
 *         Homepage: http://code.google.com/p/redjey/<br>
 *         Feedback: ykrapiva@gmail.com<br>
 *         Copyright (C) 2008 Yevgen Krapiva<br>
 */

import com.incrediblesoft.redjey.event.FocusEvent;
import com.incrediblesoft.redjey.event.AbstractEvent;

import javax.microedition.lcdui.Graphics;
import java.util.Vector;

public class Window extends Container
{
    protected Vector focusListeners = new Vector();
    /**
     * Focus flag
     */
    protected boolean isFocused;

    /**
     * Modal flag
     */
    protected boolean isModal;

    /**
     * Constructs default window
     */
    public Window()
    {
        this(null);
    }

    /**
     * Constructs window with specified style class name
     * @param styleClassName Style class name
     */
    public Window(String styleClassName)
    {
        super(styleClassName);
    }

    public String getName()
    {
        return "Window";
    }

    protected boolean isLightweight()
    {
        return false;
    }

    protected boolean isInitiallyVisible()
    {
        return false;
    }

    protected void processEvent(AbstractEvent event)
    {
        switch(event.getEventType())
        {
            case AbstractEvent.FOCUS_EVENT:
                processFocusEvent((FocusEvent) event);
                break;

            default:
                super.processEvent(event);
        }
    }

    /**
     * Processes focus events
     * @param event Focus event
     */
    protected void processFocusEvent(FocusEvent event)
    {
        switch(event.getID())
        {
            case FocusEvent.FOCUS_GAINED:
                this.isFocused = true;
                // System.out.println(""+toString()+" focus GAINED");
                break;

            case FocusEvent.FOCUS_LOST:
                this.isFocused = false;
                //System.out.println(""+toString()+" focus LOST");
                break;
        }
    }

    public void setVisible(boolean visible)
    {
        super.setVisible(visible);
        if (visible)
            UIManager.insertWindowAt(this, 0);
        else
            UIManager.removeWindow(this);        
    }

    /**
     * Sets modality flag
     * @param modal true - window always on top, otherwise - window can go background.
     */
    public void setModal(boolean modal)
    {
        this.isModal = modal;
    }

    /**
     * Returns whether whit window can go background or not.
     * @return
     */
    public boolean isModal()
    {
        return isModal;
    }

    /**
     * Maximizes this window
     */
    public void setFullScreen()
    {
        setSize(UIManager.getCanvas().getWidth(), UIManager.getCanvas().getHeight());
    }

    /*public void setState(int state)
    {
        System.out.println(toString()+" state is changing to "+state);
        super.setState(state);
    }*/

    public void paint(Graphics g)
    {
        super.paint(g);
    }

    public void addFocusListener(FocusListener listener)
    {
        focusListeners.addElement(listener);
    }

    protected void dispatchEventToListeners(FocusEvent event)
    {
        for (int i=0; i<focusListeners.size(); i++)
        {
            FocusListener listener = (FocusListener) focusListeners.elementAt(i);
            switch (event.getID())
            {
                case FocusEvent.FOCUS_GAINED:
                    listener.focusGained(event);
                    break;
                case FocusEvent.FOCUS_LOST:
                    listener.focusLost(event);
                    break;
            }
        }
    }

    public void removeFocusListener(FocusListener listener)
    {
        focusListeners.removeElement(listener);
    }
}
